package AntiCaptcha;

use strict;
use IO::Socket;
use Carp qw(croak);

sub new
{
 my $self = $_[0];
 my $obj = bless 
 {
 verbose => 0,
 phrase => 0,
 regsense => 0,
 numeric => 0,
 min_len => 0,
 max_len => 0,
 }, $self;
 
 croak "AntiCaptcha: please specify host" unless defined $_[1];
 croak "AntiCaptcha: please specify key" unless defined $_[2];
 
 $obj->{'host'} = $_[1]; 
 $obj->{'key'} = $_[2];

 return $obj;
}

sub sock 
{
 my $sock = new IO::Socket::INET 
		(
		PeerAddr => $_[0],
		PeerPort => 80, 
		PeerProto => 'tcp', 
		TimeOut => 10
		) or warn "Can't connect to anti-captcha\n";
 
 return $sock;
}

sub Recognize
{
 my $self = shift;
 my $img = $_[0];
 my $type = $_[1]; # 0 -file; 1 - data
 
 my $file = undef;
 if(!$type)
	{
	 undef $/;
	 open F, "<", $img || warn $img."-".$!."\n";
	 $file = <F>;
	 close F;
	}
 else
	{
	 $file = $img;
	}
 
 my $data =
 "--2011\n".
 "Content-Disposition: form-data; name=\"method\"\n\n".
 "post\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"key\"\n\n".
 $self->{'key'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"phrase\"\n\n".
 $self->{'phrase'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"regsense\"\n\n".
 $self->{'regsense'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"numeric\"\n\n".
 $self->{'numeric'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"min_len\"\n\n".
 $self->{'min_len'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"max_len\"\n\n".
 $self->{'max_len'}."\n".
 "--2011\n".
 "Content-Disposition: form-data; name=\"file\"; filename=\"image.jpg\"\n".
 "Content-Type: data\n\n".
 $file."\n".
 "--2011--";
 
 my $request =
 "POST /in.php HTTP/1.0\n".
 "Host: ".$self->{'host'}."\n".
 "Content-Type: multipart/form-data; boundary=2011\n".
 "Content-Length: ".length($data)."\n\n".$data;
 
 my $sock = sock($self->{'host'});
 if($sock)
	{
	 print $sock $request;
	 read $sock, my $answ, 1024;
	 close $sock;
 
	if($answ =~ /\r\n\r\n(.+)/s)
		{
		 if($1 =~ /OK\|(\d+)/)
			{
			 return $1
			}
		 elsif($1 =~ /ERROR_KEY_DOES_NOT_EXIST /)
			{
			 croak "Wrong anti-captcha key\n"
			}
		 elsif($1 =~ /ERROR_TOO_BIG_CAPTCHA_FILESIZE/)
			{
			 warn "Too big captcha file\n" if $self->{'verbose'};
			}
		 elsif($1 =~ /ERROR_ZERO_CAPTCHA_FILESIZE/)
			{
			 warn "Zero captcha file\n" if $self->{'verbose'};
			 return 0;
			}
		 elsif($1 =~ /ERROR_WRONG_FILE_EXTENSION/)
			{
			 warn "Wrong captcha file extension\n" if $self->{'verbose'};
			}
		 elsif($1 =~ /ERROR_NO_SLOT_AVAILABLE/)
			{
			 warn "No slot available\n" if $self->{'verbose'};
			 $self->Recognize($img);
			}
		}
	else
		{
		 warn "Something wrong with response\n" if $self->{'verbose'};
		}
	}
 else
	{
	 warn "Can't connect to AntiCaptcha\n" if $self->{'verbose'};
	}
 return 0;
}

sub GetStatus
{
 my $self = shift;
 my $id = $_[0];
 
 croak "Specified ID must be numeric" unless $id =~ /^\d+/;
 
 my $request =
 "GET /res.php?key=".$self->{'key'}."&action=get&id=".$id." HTTP/1.0\n".
 "Host: ".$self->{'host'}."\n\n";
 
 my $sock = sock($self->{'host'});
 if($sock)
	{
	 print $sock $request;
	 read $sock, my $answ, 1024;
	 close $sock;
 
	 if($answ =~ /\r\n\r\n(.+)/s)
		{
		 if($1 =~ /OK\|(.+)/s)
			{
			 return $1
			}
		 elsif($1 =~ /CAPCHA_NOT_READY/)
			{
			 warn "Waiting for recognition\n" if $self->{'verbose'};
			}
		 elsif($1 =~ /ERROR_KEY_DOES_NOT_EXIST/)
			{
			 croak "Wrong anti-captcha key\n"
			}
		 elsif($1 =~ /ERROR_WRONG_ID_FORMAT/)
			{
			 warn "Wrong captcha id format\n" if $self->{'verbose'};
			 return 2
			}
		 elsif($1 =~ /ERROR_NO_SUCH_CAPCHA_ID/)
			{
			 warn "No such captcha id\n" if $self->{'verbose'};
			 return 2
			}
		}
	 else
		{
		 warn "Something wrong with response\n" if $self->{'verbose'};
		}
	}
 else
	{
	 warn "Can't connect to AntiCaptcha\n" if $self->{'verbose'};
	}
 return 0;
}

1;